<?php
require_once(dirname(__FILE__) . '/common.php');
require_once(dirname(__FILE__) . '/util.php');
require_once(dirname(__FILE__) . '/EazyLog.php');
define('SQL_DEL_SITE', 'S.DEL_DATE IS NOT NULL');
define('SQL_SHOW_SITE', 'S.SHOW = 1');
define('SQL_ERR_SITE', 'S.STATUS = 9');
define('SQL_OUTDATE_SITE', "(S.DEADLINE IS NOT NULL AND S.DEADLINE < JULIANDAY('now'))");
define('SQL_LIVE_SITE', "S.DEL_DATE IS NULL AND S.SHOW = 1 AND S.STATUS != 9 AND (S.DEADLINE IS NULL OR S.DEADLINE > JULIANDAY('now'))");
define('SQL_SAMPLE_SITE', "S.USER_ID = '".ADMIN_ID."'");
define('SQL_OUT_SITE', 'S.ID IN (%s)');
define('SQL_MARK_SITE', 'S.ID IN (%s)');
define('SQL_STEALTH_SITE', 'S.STEALTH = 1');
define('SQL_NEW_SITE', "S.REG_DATE > JULIANDAY('now','-%d days')");
define('SQL_UPD_SITE', "S.SORTABLE_DATE > JULIANDAY('now','-%d days')");
define('SQL_OLD_SITE', "S.SORTABLE_DATE < JULIANDAY('now','-1 years')");
define('SQL_MOVE_SITE', "S.MOVED > JULIANDAY('now','-1 months')");
class DataBase {
	protected $conn = null;
	protected $log = null;
	/***********************************************************
	 * コンストラクタ
	 ***********************************************************/
	public function __construct() {
		$log = new EazyLog();
	}
	/***********************************************************
	 * デストラクタ
	 ***********************************************************/
	public function __destruct() {
		$this->disconnect();
	}
	/***********************************************************
	 * データベースに接続する。一度のHTTPリクエストで接続は一度。
	 * 例外が発生した場合はメッセージを表示して処理を終了する。
	 *
	 * @param string $path パスつきファイル名
	 ***********************************************************/
	public function connect($path=SQLITE_DATA_PATH) {
		if ($this->conn === null) {
			try{
				$this->conn = new PDO('sqlite:'.$path, null, null);
				$this->conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
				$this->conn->setAttribute(PDO::ATTR_TIMEOUT, 8);
			} catch (PDOException $e) {
				if (DEBUG) {
					$log->writeDebug('[' . __METHOD__ . '] '.$e->getMessage());
					exit;
				} else {
					$log->writeError('[' . __METHOD__ . '] '.$e->getMessage());
					abort('エラーが発生しました。お手数ですが管理者宛にご一報ください。');
				}
			}
		}
		return $this->conn;
	}
	/***********************************************************
	 * データベースから切断する。
	 ***********************************************************/
	public function disconnect() {
		$this->conn = null;
	}
	/***********************************************************
	 * エラーが発生したかを返す。
	 *
	 * @return エラー発生時はtrue、それ以外はfalse
	 ***********************************************************/
	public function isError() {
		if (!$this->conn) return false;
		return ($this->conn->errorCode()!=='00000');
	}
	/***********************************************************
	 * トランザクションを開始する。
	 *
	 * @return PDO::beginTransaction の戻り値
	 ***********************************************************/
	public function beginTransaction() {
		if (!$this->conn) return false;
		return $this->conn->beginTransaction();
	}
	/***********************************************************
	 * SQLを実行する。
	 *
	 * @param string SQL
	 * @return 実行結果
	 ***********************************************************/
	public function execSql($sql) {
		if (!$this->conn) return false;
		return $this->conn->exec($sql);
	}
	/***********************************************************
	 * コミットする。
	 *
	 * @return PDO::commit() の戻り値
	 ***********************************************************/
	public function commit() {
		if (!$this->conn) return false;
		return $this->conn->commit();
	}
	/***********************************************************
	 * ロールバックする。
	 *
	 * @return PDO::rollBack() の戻り値
	 ***********************************************************/
	public function rollback() {
		if (!$this->conn) return false;
		return $this->conn->rollBack();
	}
	/***********************************************************
	 * エラー情報を返す。
	 *
	 * @return PDO::errorInfo()  の戻り値
	 ***********************************************************/
	public function errorInfo() {
		if (!$this->conn) return false;
		return $this->conn->errorInfo();
	}
	/***********************************************************
	 * ロールバックし、dieでスクリプトを終了する
	 *
	 * @param string $msg 異常終了メッセージ
	 ***********************************************************/
	public function rollbackAndDie($msg) {
		$result = $this->conn->rollBack();
		if ($result === false) abort($msg.'トランザクションの破棄に失敗しました。');
		abort($msg);
	}
	/***********************************************************
	 * クエリを実行し、結果を配列に格納して返す
	 *
	 * @param string $sql SQL
	 * @param array $paramArray SQLに埋め込む値の配列。不要ならnull
	 * @return 結果の１行を１要素として格納した配列
	 ***********************************************************/
	public function fetchAll($sql, $paramArray) {
		$result = null;
		if (!$this->conn) return $result;
		try {
			$sth = $this->conn->prepare($sql);
			$status = $sth->execute($paramArray);
			if ($status) {
				$result = $sth->fetchAll();
			} else {
				if (DEBUG) echo '<p>No Data: ' . $sql . '<br>' . $sth->errorInfo() . "</p>\n";
			}
		} catch (PDOException $e) {
			if (DEBUG) echo '<p>[' . __METHOD__ . '] ' . $e->getMessage() . "</p>\n";
		}
		return ($result ? $result : array());
	}
	/***********************************************************
	 * クエリを実行し、結果の先頭レコードの先頭カラムを返す
	 *
	 * @param string $sql SQL
	 * @param array $paramArray SQLに埋め込む値の配列。不要ならnull
	 * @return 結果の先頭レコードの先頭カラム。なければnull
	 ***********************************************************/
	public function fetchSingle($sql, $paramArray) {
		$val = null;
		if (!$this->conn) return $val;
		try {
			$sth = $this->conn->prepare($sql);
			$status = $sth->execute($paramArray);
			if ($status) {
				$val = $sth->fetchColumn();
				$sth->closeCursor();
			} else {
				if (DEBUG) echo '<p>No Data: ' . $sql . '<br>' . $sth->errorInfo() . "</p>\n";
			}
		} catch (PDOException $e) {
			if (DEBUG) echo '<p>[' . __METHOD__ . '] ' . $e->getMessage() . "</p>\n";
		}
		return $val;
	}
	/***********************************************************
	 * クエリを実行し、結果の各レコードの指定カラムのみ配列にして返す
	 *
	 * @param string $sql SQL
	 * @param array $paramArray SQLに埋め込む値の配列。不要ならnull
	 * @param string $colName 取得するカラム名。大文字小文字区別あり
	 * @return 結果の各レコードの指定カラムを１要素とする配列
	 ***********************************************************/
	public function fetchSingleList($sql, $paramArray, $colName) {
		$result = array();
		if (!$this->conn) return $result;
		try {
			$sth = $this->conn->prepare($sql);
			$status = $sth->execute($paramArray);
			if ($status) {
				while($row = $sth->fetch()) {
					$result[] = $row[$colName];
				}
			} else {
				if (DEBUG) echo '<p>No Data: ' . $sql . '<br>' . $sth->errorInfo() . "</p>\n";
			}
		} catch (PDOException $e) {
			if (DEBUG) echo '<p>[' . __METHOD__ . '] ' . $e->getMessage() . "</p>\n";
		}
		return $result;
	}
}
